import 'dart:convert';

import 'common/pagingquery.dart';
import 'common/pagingresult.dart';
import 'common/userid.dart';
import 'common/useridlist.dart';
import 'communities/public.dart';
import 'communities/queries/public.dart';
import 'src/native_bridge.dart';
import 'src/utils.dart';
import 'usermanagement/user.dart';

class Communities {
  ///
  /// Add a friend to current user, if operation succeeds, they both became friends.
  /// If try to add a user who is already a friend, `success` callback will be called,
  /// but user will be added to friends list only once and friends count won't be increased.
  ///
  /// [ids]   List of user identifiers to add as friend.
  ///
  static Future<int> addFriends(UserIdList ids) {
    return NativeBridge.async(
            'Communities.addFriends', jsonEncode(ids.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Remove users from friends list.
  /// If trying to remove a user who is not a friend, success callback will be called,
  /// and user's friends count won't be decreased.
  ///
  /// [ids]   List of user identifiers to remove from friends list.
  ///
  static Future<int> removeFriends(UserIdList ids) {
    return NativeBridge.async(
            'Communities.removeFriends', jsonEncode(ids.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Check if users are friend of current user.
  ///
  /// [ids]   List of user identifiers.
  ///
  static Future<Map<String, bool>> areFriends(UserIdList ids) {
    return NativeBridge.async(
            'Communities.areFriends', jsonEncode(ids.toJSON()))
        .then((result) {
      Map<String, bool> resultMap = {};
      jsonDecode(result).forEach((key, value) {
        resultMap[key] = value;
      });
      return resultMap;
    });
  }

  ///
  /// Check if user is friend of current user.
  ///
  /// [id]  User identifier.
  ///
  static Future<bool> isFriend(UserId id) {
    return NativeBridge.async('Communities.isFriend', jsonEncode(id.toJSON()))
        .then((result) => jsonDecode(result)['result'] == true);
  }

  ///
  /// Return number of friends of a user.
  ///
  /// [query]   Query parameters.
  ///
  static Future<int> getFriendsCount(FriendsQuery query) {
    return NativeBridge.async(
            'Communities.getFriendsCount', jsonEncode(query.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Get a list of friends of a user.
  ///
  /// [pagingQuery]   Query parameters.
  ///
  static Future<PagingResult<User>> getFriends(
      PagingQuery<FriendsQuery> pagingQuery) {
    return NativeBridge.async(
            'Communities.getFriends', jsonEncode(pagingQuery.toJSON()))
        .then((result) => new PagingResult.fromJSON(jsonDecode(result),
            (Map<String, dynamic> raw) => User.fromJson(raw)));
  }

  ///
  /// Get a list of suggested friends of current user.
  ///
  /// [pagingQuery]   Query parameter.
  ///
  static Future<PagingResult<SuggestedFriend>> getSuggestedFriends(
      PagingQuery pagingQuery) {
    return NativeBridge.async(
            'Communities.getSuggestedFriends', jsonEncode(pagingQuery.toJSON()))
        .then((result) => new PagingResult.fromJSON(jsonDecode(result),
            (Map<String, dynamic> raw) => SuggestedFriend.fromJson(raw)));
  }

  ///
  /// Set the provided users as friends of current user.
  /// Important:
  /// Existing friends will be removed!
  ///
  /// [ids] List of user identifiers.
  ///
  static Future<int> setFriends(UserIdList ids) {
    return NativeBridge.async(
            'Communities.setFriends', jsonEncode(ids.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Find users.
  ///
  /// [pagingQuery]  Search parameter.
  ///
  static Future<PagingResult<User>> findUsers(
      PagingQuery<UsersQuery> pagingQuery) {
    return NativeBridge.async(
            'Communities.getUsers', jsonEncode(pagingQuery.toJSON()))
        .then((result) {
      return new PagingResult.fromJSON(
          jsonDecode(result), (Map<String, dynamic> raw) => User.fromJson(raw));
    });
  }

  ///
  /// Get multiple users by identifiers.
  ///
  /// [ids]   List of user identifiers.
  ///
  static Future<Map<String, User>> getUsers(UserIdList ids) {
    return NativeBridge.async(
            'Communities.getUsersByIds', jsonEncode(ids.toJSON()))
        .then((result) {
      Map<String, dynamic> jsonMap = jsonDecode(result);
      return jsonMap.map((key, value) => MapEntry(key, User.fromJson(value)));
    });
  }

  ///
  /// Return number of user matching the provided query.
  ///
  /// [query]   Query parameter.
  ///
  static Future<int> getUsersCount(UsersQuery query) {
    return NativeBridge.async(
            'Communities.getUsersCount', jsonEncode(query.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Follow a community entity, like topic or user.
  ///
  /// [query]   Query parameter.
  ///
  static Future<int> follow(FollowQuery query) {
    return NativeBridge.async('Communities.follow', jsonEncode(query.toJSON()))
        .then((result) {
      return jsonDecode(result)['result'];
    });
  }

  ///
  /// Unfollow a community entity, like topic or user.
  ///
  /// [query]   Query parameter.
  ///
  static Future<int> unfollow(FollowQuery query) {
    return NativeBridge.async(
            'Communities.unfollow', jsonEncode(query.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Check if a user follows a community entity, like topic or user.
  ///
  /// [id]      User id.
  /// [query]   Query parameter.
  ///
  static Future<Map<String, bool>> isFollowing(UserId id, FollowQuery query) {
    Map<String, dynamic> isFollowingBody = {
      'userId': id.toJSON(),
      'query': query.toJSON()
    };
    return NativeBridge.async(
            'Communities.isFollowing', jsonEncode(isFollowingBody))
        .then((result) {
      Map<String, dynamic> jsonMap = jsonDecode(result);
      return jsonMap.map((key, value) => MapEntry(key, value));
    });
  }

  ///
  /// Get followers of a community entity, like topic or user.
  ///
  /// [pagingQuery]   Query parameter.
  ///
  static Future<PagingResult<User>> getFollowers(
      PagingQuery<FollowersQuery> pagingQuery) {
    return NativeBridge.async(
            'Communities.getFollowers', jsonEncode(pagingQuery.toJSON()))
        .then((result) => new PagingResult.fromJSON(jsonDecode(result),
            (Map<String, dynamic> raw) => User.fromJson(raw)));
  }

  ///
  /// Get number of followers of a community entity, like topic or user.
  ///
  /// [query]   Query parameter.
  ///
  static Future<int> getFollowersCount(FollowersQuery query) {
    return NativeBridge.async(
            'Communities.getFollowersCount', jsonEncode(query.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Get announcements.
  ///
  /// [query]   Query parameter.
  ///
  static Future<List<Activity>> getAnnouncements(AnnouncementsQuery query) {
    return NativeBridge.async(
            'Communities.getAnnouncements', jsonEncode(query.toJSON()))
        .then((result) {
      List jsonList = jsonDecode(result);
      return jsonList.map((e) => Activity.fromJSON(e)).toList();
    });
  }

  ///
  /// Get activities.
  ///
  /// [pagingQuery]   Query parameter.
  ///
  static Future<PagingResult<Activity>> getActivities(
      PagingQuery<ActivitiesQuery> pagingQuery) {
    return NativeBridge.async(
            'Communities.getActivities', jsonEncode(pagingQuery.toJSON()))
        .then((result) => new PagingResult.fromJSON(jsonDecode(result),
            (Map<String, dynamic> raw) => Activity.fromJSON(raw)));
  }

  ///
  /// Get a single activity.
  ///
  /// [id]    Activity id.
  ///
  static Future<Activity> getActivity(String id) {
    return NativeBridge.async('Communities.getActivity', id)
        .then((result) => Activity.fromJSON(jsonDecode(result)));
  }

  ///
  /// Post activity content to the specified target.
  ///
  /// [content]   Content to post.
  /// [target]    Where to post the content.
  ///
  static Future<Activity> postActivity(
      ActivityContent content, PostActivityTarget target) {
    Map<String, dynamic> postActivityBody = {
      'target': target.toJSON(),
      'content': content.toJSON()
    };
    return NativeBridge.async(
            'Communities.postActivity', jsonEncode(postActivityBody))
        .then((result) => Activity.fromJSON(jsonDecode(result)));
  }

  ///
  /// Update existing activity with provided content.
  ///
  /// [id]      Activity id to update.
  /// [content] New content.
  ///
  static Future<Activity> updateActivity(String id, ActivityContent content) {
    Map<String, dynamic> updateActivityBody = {
      'target': id,
      'content': content.toJSON()
    };
    return NativeBridge.async(
            'Communities.updateActivity', jsonEncode(updateActivityBody))
        .then((result) => Activity.fromJSON(jsonDecode(result)));
  }

  ///
  /// Add reaction to an activity.
  ///
  /// [reaction]    Reaction to add.
  /// [activityId]  To add the reaction to.
  ///
  static Future addReaction(String reaction, String activityId) {
    Map<String, dynamic> addReactionBody = {
      'reaction': reaction,
      'activityId': activityId
    };
    return NativeBridge.async(
        'Communities.addReaction', jsonEncode(addReactionBody));
  }

  ///
  /// Remove a reaction from an activity.
  ///
  /// [reaction]    Reaction to remove.
  /// [activityId]  To remove the reaction from.
  ///
  static Future removeReaction(String reaction, String activityId) {
    Map<String, dynamic> removeReactionBody = {
      'reaction': reaction,
      'activityId': activityId
    };
    return NativeBridge.async(
        'Communities.removeReaction', jsonEncode(removeReactionBody));
  }

  ///
  /// Get reactions.
  ///
  /// [pagingQuery]   Query parameters.
  ///
  static Future<PagingResult<UserReactions>> getReactions(
      PagingQuery<ReactionsQuery> pagingQuery) {
    return NativeBridge.async(
            'Communities.getReactions', jsonEncode(pagingQuery.toJSON()))
        .then((result) => new PagingResult.fromJSON(jsonDecode(result),
            (Map<String, dynamic> raw) => UserReactions.fromJSON(raw)));
  }

  ///
  /// Report an activity.
  ///
  /// [id]            Activity id to report.
  /// [reason]        Reason of reporting.
  /// [explanation]   Explanation of reason.
  ///
  static Future reportActivity(String id, int reason, String explanation) {
    Map<String, dynamic> reportActivityBody = {
      'activityId': id,
      'reason': reason,
      'explanation': explanation,
    };
    return NativeBridge.async(
        'Communities.reportActivity', jsonEncode(reportActivityBody));
  }

  ///
  /// Remove activities.
  ///
  /// [query]   Query parameters.
  ///
  static Future removeActivities(RemoveActivitiesQuery query) {
    return NativeBridge.async(
        'Communities.removeActivities', jsonEncode(query.toJSON()));
  }

  ///
  /// Find tags.
  ///
  /// [query]   Query parameters.
  ///
  static Future<List<String>> getTags(TagsQuery query) {
    return NativeBridge.async('Communities.getTags', jsonEncode(query.toJSON()))
        .then((result) => Utils.createStringList(jsonDecode(result)));
  }

  ///
  /// Get a single topic.
  ///
  /// [id]    Topic id.
  ///
  static Future<Topic> getTopic(String id) {
    return NativeBridge.async('Communities.getTopic', id)
        .then((result) => Topic.fromJSON(jsonDecode(result)));
  }

  ///
  /// Find topics.
  ///
  /// [pagingQuery]   Query parameter.
  ///
  static Future<PagingResult<Topic>> getTopics(
      PagingQuery<TopicsQuery> pagingQuery) {
    return NativeBridge.async(
            'Communities.getTopics', jsonEncode(pagingQuery.toJSON()))
        .then((result) => new PagingResult.fromJSON(jsonDecode(result),
            (Map<String, dynamic> raw) => Topic.fromJSON(raw)));
  }

  ///
  /// Get number of topics.
  ///
  /// [query]   Query parameter.
  ///
  static Future<int> getTopicsCount(TopicsQuery query) {
    return NativeBridge.async(
            'Communities.getTopicsCount', jsonEncode(query.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }
}
