import 'dart:convert';

import 'common/action.dart';
import 'src/native_bridge.dart';
import 'current_user.dart';
import 'usermanagement/identity.dart';

class GetSocial {
  /// Returns SDK version
  static get version => "7.1.0";

  /// Initialize GetSocial SDK if automatic initialization is disabled.
  static void init() {
    NativeBridge.sync("GetSocial.init", "");
  }

  /// Current SDK language
  static Future<String> getLanguage() {
    return NativeBridge.sync("GetSocial.getLanguage", "")
        .then((value) => jsonDecode(value)["result"]);
  }

  /// Set SDK language.
  ///
  /// [language] Language to use, check {@link LanguageCodes} for supported languages.
  ///
  static Future<bool> setLanguage(String language) {
    return NativeBridge.sync("GetSocial.setLanguage", language)
        .then((value) => jsonDecode(value)["result"]);
  }

  ///
  /// Set a [listener] Function, which will be invoked after SDK initialized.
  /// Function will be executed immediately, if SDK is already initialized.
  ///
  static void addOnInitializedListener(Function listener) {
    NativeBridge.registerListener(
        "GetSocial.addOnInitializedListener", (any) => listener());
  }

  ///
  /// Current user.
  ///
  /// Returns current user or null if SDK is not initialized.
  ///
  static Future<CurrentUser> get currentUser {
    return NativeBridge.sync("GetSocial.getCurrentUser", null)
        .then((value) => CurrentUser.fromJson(jsonDecode(value)));
  }

  ///
  /// Switches the current user with the [User] corresponding to the details provided.
  ///
  /// [identity] Identity to be switched to.
  ///
  static Future switchUser(Identity identity) {
    return NativeBridge.async(
        "GetSocial.switchUser", jsonEncode(identity.toJSON()));
  }

  ///
  /// Reset current user and create a new anonymous one.
  ///
  static Future resetUser() {
    return NativeBridge.async("GetSocial.resetUser", null);
  }

  ///
  /// Add listener to be notified when the user was changed.
  /// The listener action is executed on the main thread, so be careful with operations,
  /// that you put inside on {@link OnCurrentUserChangedListener#onUserChanged(CurrentUser)} method.
  /// Listener will be invoked when:
  /// SDK initialization is finished;
  /// - [GetSocial.switchUser] method was called and user was successfully changed.
  ///
  /// [listener] Listener to be notified.
  ///
  static void addOnCurrentUserChangedListener(Function(CurrentUser) listener) {
    NativeBridge.registerListener("GetSocial.addOnCurrentUserChangedListener",
        (user) => {listener(CurrentUser.fromJson(jsonDecode(user)))});
  }

  /// Handle the provided action if it's supported.
  ///
  /// [action]  action to handle.
  ///
  static void handleAction(GetSocialAction action) {
    if (action.type == 'open_invites' || action.type == 'open_activity') {
      return;
    }
    NativeBridge.sync("GetSocial.handleAction", jsonEncode(action.toJSON()));
  }
}
