import 'dart:convert';

import 'common/pagingquery.dart';
import 'common/pagingresult.dart';
import 'src/native_bridge.dart';
import 'invites/public.dart';
import 'common/userid.dart';

class Invites {
  ///
  /// Get the list of available invite channels. Contains only channels enabled on the Dashboard and available on the device.
  ///
  static Future<List<InviteChannel>> getAvailableChannels() {
    return NativeBridge.async("Invites.getAvailableChannels", '').then(
        (value) => (jsonDecode(value) as List)
            .map((e) => InviteChannel.fromJSON(e))
            .toList());
  }

  ///
  /// Invite friends via a specified invite channel.
  ///
  /// [content]               Custom content to override the default content provided from the Dashboard.
  /// [channelId]             The channel through which the invite will be sent, one of the constants defined in {@link InviteChannelIds}.
  /// [onSuccess]             Called when the invite process is complete.
  /// [onCancel]              Called if a user canceled an invite.
  /// [onFailure]             Called if operation failed.
  ///
  static void send(InviteContent content, String channelId,
      Function() onSuccess, Function() onCancel, Function(Error) onFailure) {
    Map<String, dynamic> inviteBody = {
      'channelId': channelId,
      'content': content == null ? null : content.toJSON()
    };
    NativeBridge.async("Invites.send", jsonEncode(inviteBody)).then((value) {
      if (value == null) {
        onSuccess();
        return;
      }
      jsonDecode(value)['result'] == 'cancel' ? onCancel() : onSuccess();
    });
  }

  ///
  /// Create invite with localized values and a link.
  ///
  /// [content]     Custom content to override the default content provided from the Dashboard.
  ///
  static Future<Invite> create(InviteContent content) {
    return NativeBridge.async("Invites.create",
            jsonEncode(content == null ? null : content.toJSON()))
        .then((value) => Invite.fromJSON(jsonDecode(value)));
  }

  ///
  /// Creates a Smart Link with user referral data attached used for Smart Invites.
  ///
  /// [content]   Invite Content
  ///
  static Future<String> createURL(InviteContent content) {
    return NativeBridge.async("Invites.createLink",
            jsonEncode(content == null ? null : content.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  ///
  /// Set listener to receive referral data if there is any.
  ///
  /// [listener]    To be called with a new referral data.
  ///
  static void setOnReferralDataReceivedListener(
      Function(ReferralData) listener) {
    NativeBridge.registerListener(
        "Invites.setOnReferralDataReceivedListener",
        (result) => listener(
            result == null ? '' : ReferralData.fromJSON(jsonDecode(result))));
  }

  ///
  /// Returns list of users who were referred by this user for a specific event.
  ///
  /// [pagingQuery]   Users query.
  ///
  static Future<PagingResult<ReferralUser>> getReferredUsers(
      PagingQuery<ReferralUsersQuery> pagingQuery) {
    return NativeBridge.async(
            'Invites.getReferredUsers', jsonEncode(pagingQuery.toJSON()))
        .then((value) => new PagingResult.fromJSON(jsonDecode(value),
            (Map<String, dynamic> raw) => ReferralUser.fromJson(raw)));
  }

  ///
  /// Returns list of users who are referrers for this user for a specific event.
  ///
  /// [pagingQuery]     Users query.
  ///
  static Future<PagingResult<ReferralUser>> getReferrerUsers(
      PagingQuery<ReferralUsersQuery> pagingQuery) {
    return NativeBridge.async(
            'Invites.getReferrerUsers', jsonEncode(pagingQuery.toJSON()))
        .then((value) => new PagingResult.fromJSON(jsonDecode(value),
            (Map<String, dynamic> raw) => ReferralUser.fromJson(raw)));
  }

  ///
  /// Sets referrer id for current user.
  ///
  /// [id]          Id of referrer user.
  /// [event]       Referrer event.
  /// [customData]  Custom key-value pairs.
  ///
  static Future setReferrer(
      UserId id, String event, Map<String, String> customData) {
    Map<String, dynamic> body = {
      'userId': id.toJSON(),
      'eventName': event,
      'customData': customData == null ? new Map() : customData
    };
    return NativeBridge.async('Invites.setReferrer', jsonEncode(body));
  }
}
